#!/bin/bash
#
# dp-interop-client.sh, this script is used to:
#   1. generate request messages that conform to the schema
#      'dp-xslt-interop-protocol.wsdl',
#   2. and send the generated messages to the interop-service on DataPower
#      appliance then.
#
# There are three kinds of request messages, that is, XSLT, FFD, and XPath. The
# three different messages request the interop service to execute the XSLT, FFD,
# and XPath transformations respectively.
#

#variables for cmd options
type="xslt"
xform_pi=""
xform_input=""
xform_mime=""
zip_file=""
validate_type="schema"
expr=""
more_files=""
host=""
port=0
auth=""
#debugged=0
secured=0
msg_only=0

#the script usage
function print_usage
{
  echo -e \
    "usage: $0 [-t xslt|ffd|xpath|schema] [-z zipfile] [-x stylesheet] [-zx stylesheet] [-i input-file] [-zi input-file] [-a mime-file] [-e xpath-expr] [-v schema|wsdl] [-h hostname] [-p port] [-s] [-u user:password] [-m]\n\n" \
    "\t-t  <xslt|ffd|xpath|schema>  request type (the default is set to xslt)\n" \
    "\t-z  <zip-file>        the zip file that contains the stylesheet and (or) input files needed for transformation.\n" \
    "\t-x  <stylesheet>      the stylesheet file or the schema file\n" \
    "\t-zx <stylesheet>      the stylesheet file or the schema file that can be found in the zip file.\n" \
    "\t-i  <input-file>      the input file for transformation. Use comma to separate multiple input files.\n" \
    "\t-zi <input-file>      the input file that can be found in the zip file.\n" \
    "\t-a  <mime-file>       the SwA or Mime input file for transformation\n" \
    "\t-e  <xpath-expr>      the XPath expression\n" \
    "\t-v  <schema|wsdl>     the validation type (the default is set to schema)\n" \
    "\t-f  <other-files>     to package the additional files in the request message. Use comma to separate them.\n" \
    "\t                      For exapmle, to include the FFD files with the request message.\n" \
    "\t-h  <hostname>        hostname\n" \
    "\t-p  <port>            port number (defaults to 9990 and 9991 for HTTP and HTTPS respectively)\n" \
    "\t-s                    to send request over HTTPS\n" \
    "\t-u  <user:password>   the username and password.\n" \
    "\t                      Check the AAA Policy of Interop Test Service to see if the basic authentication is required.\n" \
    "\t-m                    to generate the request message only instead of sending the request.\n\n" \
    "[XSLT]      $0 -x transform.xsl -i input.xml -h dpbox.foo.com\n" \
    "            $0 -x transform.xsl -i input.xml -h dpbox.foo.com -p 9991 -s\n" \
    "            $0 -z artifacts.zip -zx dir1/transform.xsl -zi dir2/data.xml -h dpbox.foo.com\n\n" \
    "[XSLT-swa]  $0 -x cid:transform.xsl -a input.swa -h dpbox.foo.com\n" \
    "            $0 -x transform.xsl -a input.swa -h dpbox.foo.com\n" \
    "            $0 -x transform.xsl -i file1.xml,file2.xxx,file3.xxx -h dpbox.foo.com\n" \
    "            $0 -z artifacts.zip -x transform.xsl -zi dir/file1.xml,dir/file2.dat,dir/file3.bin -h dpbox.foo.com\n\n" \
    "[Schema]    $0 -t schema -x data-type.xsd -i input.xml -h dpbox.foo.com\n" \
    "            $0 -t schema -v wsdl -x services.wsdl -i request.xml -h dpbox.foo.com -p 2048 -s\n" \
    "            $0 -t schema -v wsdl -z artifacts.zip -zx services.wsdl -zi request.xml -h dpbox.foo.com\n\n" \
    "[FFD]       $0 -t ffd -x csv2xml.xsl -i input.csv -h dpbox.foo.com\n" \
    "            $0 -t ffd -x csv2xml.xsl -i input.csv -f input.ffd,output.ffd -h dpbox.foo.com -u user:pass\n" \
    "            $0 -t ffd -z artifacts.zip -zx dir/csv2text.xsl -zi dir/input.csv -h dpbox.foo.com -u user:pass\n\n" \
    "[XPATH]     $0 -t xpath -e \"count(//*)\" -i input.xml -h dpbox.foo.com\n" \
    >&2

   #"\t-g                   Enable the debug mode. Request the Interop Test Service to provide the probe data.\n" \
   #"\t                        1. The request will be sent over HTTPS, and\n" \
   #"\t                        2. Users must provide the basic authentication with the option '-u'.\n" \
}


boundary="540af153-f51d-4c96-a16b-d44bf0dd7925"
tmpfile="tmp.request.swa"

function prepare_schema_request
{
  echo -e '\r\n\r\n--'$boundary                                                  > $tmpfile
  echo 'Content-Type: text/xml; charset=UTF-8'                                  >> $tmpfile
  echo 'Content-Transfer-Encoding: binary'                                      >> $tmpfile
  echo 'Content-Id: <request>'                                                  >> $tmpfile
  echo ''                                                                       >> $tmpfile
  echo '<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">' >> $tmpfile
  echo '  <soap:Body>'                                                          >> $tmpfile
  echo '    <test:SchemaTest type="'$validate_type'" '                          >> $tmpfile

  if [[ $xform_pi = zip:* ]] || [[ $xform_input = zip:* ]]; then
    echo '                     zipFile="cid:'$zip_file'" '                      >> $tmpfile
  fi

  echo '        xmlns:test="http://www.datapower.com/xslt-test"> '              >> $tmpfile

  if [[ $xform_pi != zip:* ]]; then
    echo '      <Schema href="cid:'$xform_pi'"/>'                               >> $tmpfile
  else
    echo '      <Schema href="'$xform_pi'"/>'                                   >> $tmpfile
  fi

  if [[ $xform_input != zip:* ]]; then
    echo '      <XMLData href="cid:'$xform_input'"/>'                           >> $tmpfile
  else
    echo '      <XMLData href="'$xform_input'"/>'                               >> $tmpfile
  fi

  echo '    </test:SchemaTest>'                                                 >> $tmpfile
  echo '  </soap:Body>'                                                         >> $tmpfile
  echo '</soap:Envelope>'                                                       >> $tmpfile

  if [[ $xform_pi = zip:* ]] || [[ $xform_input = zip:* ]]; then
    echo -e '\r\n--'$boundary                                                   >> $tmpfile
    echo 'Content-ID: <'$zip_file'>'                                            >> $tmpfile
    echo 'Content-Type: application/zip'                                        >> $tmpfile
    echo 'Content-Transfer-Encoding: binary'                                    >> $tmpfile
    echo ''                                                                     >> $tmpfile
    more $zip_file                                                              >> $tmpfile
  fi

  if [[ $xform_pi != zip:* ]]; then
    echo -e '\r\n--'$boundary                                                   >> $tmpfile
    echo 'Content-ID: <'$xform_pi'>'                                            >> $tmpfile
    echo 'Content-Type: text/xml'                                               >> $tmpfile
    echo 'Content-Transfer-Encoding: binary'                                    >> $tmpfile
    echo ''                                                                     >> $tmpfile
    more $xform_pi                                                              >> $tmpfile
  fi

  if [[ $xform_input != zip:* ]]; then
    echo -e '\r\n--'$boundary                                                   >> $tmpfile
    echo 'Content-ID: <'$xform_input'>'                                         >> $tmpfile
    echo 'Content-Type: text/xml'                                               >> $tmpfile
    echo 'Content-Transfer-Encoding: binary'                                    >> $tmpfile
    echo ''                                                                     >> $tmpfile
    more $xform_input                                                           >> $tmpfile
  fi
  echo -e '\r\n--'$boundary'--'                                                 >> $tmpfile
}

function prepare_xslt_request
{
  tmp=""
  read_from_zip=0

  #convert xform_input into an array
  if [[ $xform_input != zip:* ]]; then
    tmp=$(echo $xform_input | tr "," "\n")
  else
    #determine the zip reference
    tmp=`echo $xform_input | cut -f2 -d:`
    tmp=$(echo $tmp | tr "," "\n")
    read_from_zip=1
  fi

  files=()
  for x in ${tmp[@]}
  do
    if [[ $read_from_zip = 0 ]] && [[ ! -f $x ]]; then
        echo ">> The input data '$x' is missing..."  >&2
        exit 1;
    fi

    files=( "${files[@]}" $x )
  done
  #echo 'file cnt:' ${#files[@]}
  #echo 'file 1st:' ${files[0]}

  echo -e '\r\n\r\n--'$boundary                                                  > $tmpfile
  echo 'Content-Type: text/xml; charset=UTF-8'                                  >> $tmpfile
  echo 'Content-Transfer-Encoding: binary'                                      >> $tmpfile
  echo 'Content-Id: <request>'                                                  >> $tmpfile
  echo ''                                                                       >> $tmpfile
  echo '<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">' >> $tmpfile
  echo '  <soap:Body>'                                                          >> $tmpfile
  echo '    <test:XSLTTest '                                                    >> $tmpfile

  if [[ $xform_pi = zip:* ]] || [[ $xform_input = zip:* ]]; then
    echo '        zipFile="cid:'$zip_file'" '                                   >> $tmpfile
  fi

  echo '        xmlns:test="http://www.datapower.com/xslt-test">'               >> $tmpfile

  if [[ $xform_pi != zip:* ]]; then
    echo '      <Stylesheet href="cid:'$xform_pi'"/>'                           >> $tmpfile
  else
    echo '      <Stylesheet href="'$xform_pi'"/>'                               >> $tmpfile
  fi

  if [[ $xform_input = zip:* ]]; then
    echo '      <XMLData href="zip:'${files[0]}'"/>'                            >> $tmpfile
  else
    echo '      <XMLData href="cid:'${files[0]}'"/>'                            >> $tmpfile
  fi
  if [[ ${#files[@]} > 1 ]]; then
    echo ">> Found multiple input files: ${files[@]}"  >&2
    for (( i=1 ; i<${#files[@]} ; i++ ))
    do
      if [[ $xform_input = zip:* ]]; then
        echo '      <Attachment href="zip:'${files[$i]}'"/>'                    >> $tmpfile
      else
        echo '      <Attachment href="cid:'${files[$i]}'"/>'                    >> $tmpfile
      fi
    done
  fi

  echo '    </test:XSLTTest>'                                                   >> $tmpfile
  echo '  </soap:Body>'                                                         >> $tmpfile
  echo '</soap:Envelope>'                                                       >> $tmpfile

  if [[ $xform_pi = zip:* ]] || [[ $xform_input = zip:* ]]; then
    echo -e '\r\n--'$boundary                                                   >> $tmpfile
    echo 'Content-ID: <'$zip_file'>'                                            >> $tmpfile
    echo 'Content-Type: application/zip'                                        >> $tmpfile
    echo 'Content-Transfer-Encoding: binary'                                    >> $tmpfile
    echo ''                                                                     >> $tmpfile
    more $zip_file                                                              >> $tmpfile
  fi

  if [[ $xform_pi != zip:* ]]; then
    echo -e '\r\n--'$boundary                                                   >> $tmpfile
    echo 'Content-ID: <'$xform_pi'>'                                            >> $tmpfile
    echo 'Content-Type: text/xml'                                               >> $tmpfile
    echo 'Content-Transfer-Encoding: binary'                                    >> $tmpfile
    echo ''                                                                     >> $tmpfile
    more $xform_pi                                                              >> $tmpfile
  fi

  #the input data
  if [[ $read_from_zip = 0 ]]; then
    echo -e '\r\n--'$boundary                                                   >> $tmpfile
    echo 'Content-ID: <'${files[0]}'>'                                          >> $tmpfile
    echo 'Content-Type: text/xml'                                               >> $tmpfile
    echo 'Content-Transfer-Encoding: binary'                                    >> $tmpfile
    echo ''                                                                     >> $tmpfile
    more ${files[0]}                                                            >> $tmpfile
  fi

  #the other input data if there is more than one
  if [[ $read_from_zip = 0 ]]; then
    for (( i=1 ; i<${#files[@]} ; i++ ))
    do
      echo -e '\r\n--'$boundary                                                 >> $tmpfile
      echo 'Content-ID: <'${files[$i]}'>'                                       >> $tmpfile
      echo 'Content-Type: application/octet-stream'                             >> $tmpfile
      echo 'Content-Transfer-Encoding: binary'                                  >> $tmpfile
      echo ''                                                                   >> $tmpfile
      more ${files[$i]}                                                         >> $tmpfile
    done
  fi

  echo -e '\r\n--'$boundary'--'                                                 >> $tmpfile
}

function prepare_xslt_request_swa
{
  #Determine MIME boundary
  #note: the character '\r' sometimes does not show up in the end of mime boundary.
  boundary=`sed -n 's/^--\(..*\)--\r*$/\1/p' < $xform_mime | \
            tail -1`  #take the outermost ending boundary in case of embedded multipart
  echo ">> Found the MIME boundar '$boundary' in '$xform_mime'."  >&2

  if [[ $xform_pi != cid:* ]]; then
    #read the stylesheet from the local file system
    echo ">> Reading the stylesheet from the local file system..."  >&2
    read_from_local=1
  else
    #read the stylesheet from the Mime file
    echo ">> Reading the stylesheet from the Mime message..."  >&2
    read_from_local=0
    #determine the cid reference
    xform_pi=`echo $xform_pi | cut -f2 -d:`
  fi

  echo -e '\r\n\r\n--'$boundary                                                  > $tmpfile
  echo 'Content-Type: text/xml; charset=UTF-8'                                  >> $tmpfile
  echo 'Content-Transfer-Encoding: binary'                                      >> $tmpfile
  echo 'Content-Id: <request>'                                                  >> $tmpfile
  echo ''                                                                       >> $tmpfile
  echo '<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">' >> $tmpfile
  echo '  <soap:Body>'                                                          >> $tmpfile
  echo '    <test:XSLTTest xmlns:test="http://www.datapower.com/xslt-test">'    >> $tmpfile
  echo '      <Stylesheet href="cid:'$xform_pi'"/>'                             >> $tmpfile

  #instead of listing all attachments, use 'cid:*' instead
  #echo '      <XMLData href="cid:*"/>'                                         >> $tmpfile

  #retrieve all the content-ids except the stylesheet
  CIDs_wo_pi=`sed -n 's/^content-id: <\(..*\)>\r*$/\1/pi' < $xform_mime | sed '/^'$xform_pi'$/d'`
  count=1
  for i in $CIDs_wo_pi; do
    if [[ $count = 1 ]]; then
      echo '      <XMLData href="cid:'$i'"/>'                                   >> $tmpfile
      echo ">> Found the first input xml '$i' in the MIME file."                >&2
      count=count+1
    else
      echo '      <Attachment href="cid:'$i'"/>'                                >> $tmpfile
      echo ">> Found one attachment '$i' in the MIME file."                     >&2
    fi
  done

  echo '    </test:XSLTTest>'                                                   >> $tmpfile
  echo '  </soap:Body>'                                                         >> $tmpfile
  echo '</soap:Envelope>'                                                       >> $tmpfile
  echo -e '\r\n--'$boundary                                                     >> $tmpfile

  #write out the stylesheet from the local file system
  if [[ $read_from_local = 1 ]]; then
    echo 'Content-ID: <'$xform_pi'>'                                            >> $tmpfile
    echo 'Content-Type: text/xml'                                               >> $tmpfile
    echo 'Content-Transfer-Encoding: binary'                                    >> $tmpfile
    echo ''                                                                     >> $tmpfile
    more $xform_pi                                                              >> $tmpfile
    echo -e '\r\n--'$boundary                                                   >> $tmpfile
  fi

  #append the swa file without the preamble.
  sed '1,/^--'$boundary'/d' < $xform_mime                                       >> $tmpfile
}

function prepare_ffd_request
{
  #convert more_files into an array
  tmp=$(echo $more_files | tr "," "\n")
  files=()
  for x in ${tmp[@]}
  do
    if [[ ! -f $x ]]; then
        echo ">> The file '$x' is missing..."  >&2
        exit 1;
    fi

    files=( "${files[@]}" $x )
  done
  #echo 'file cnt:' ${#files[@]}
  #echo 'file 1st:' ${files[0]}

  echo -e '\r\n\r\n--'$boundary                                                  > $tmpfile
  echo 'Content-Type: text/xml; charset=UTF-8'                                  >> $tmpfile
  echo 'Content-Transfer-Encoding: binary'                                      >> $tmpfile
  echo 'Content-Id: <request>'                                                  >> $tmpfile
  echo ''                                                                       >> $tmpfile
  echo '<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">' >> $tmpfile
  echo '  <soap:Body>'                                                          >> $tmpfile
  echo '    <test:FFDTest '                                                     >> $tmpfile

  if [[ $xform_pi = zip:* ]] || [[ $xform_input = zip:* ]]; then
    echo '        zipFile="cid:'$zip_file'" '                                   >> $tmpfile
  fi

  echo '        xmlns:test="http://www.datapower.com/xslt-test">'               >> $tmpfile

  if [[ $xform_pi != zip:* ]]; then
    echo '      <Stylesheet href="cid:'$xform_pi'"/>'                           >> $tmpfile
  else
    echo '      <Stylesheet href="'$xform_pi'"/>'                               >> $tmpfile
  fi

  if [[ $xform_input != zip:* ]]; then
    echo '      <BinaryData href="cid:'$xform_input'"/>'                        >> $tmpfile
  else
    echo '      <BinaryData href="'$xform_input'"/>'                            >> $tmpfile
  fi

  echo '    </test:FFDTest>'                                                    >> $tmpfile
  echo '  </soap:Body>'                                                         >> $tmpfile
  echo '</soap:Envelope>'                                                       >> $tmpfile

  if [[ $xform_pi = zip:* ]] || [[ $xform_input = zip:* ]]; then
    echo -e '\r\n--'$boundary                                                   >> $tmpfile
    echo 'Content-ID: <'$zip_file'>'                                            >> $tmpfile
    echo 'Content-Type: application/zip'                                        >> $tmpfile
    echo 'Content-Transfer-Encoding: binary'                                    >> $tmpfile
    echo ''                                                                     >> $tmpfile
    more $zip_file                                                              >> $tmpfile
  fi

  if [[ $xform_pi != zip:* ]]; then
    echo -e '\r\n--'$boundary                                                   >> $tmpfile
    echo 'Content-ID: <'$xform_pi'>'                                            >> $tmpfile
    echo 'Content-Type: text/xml'                                               >> $tmpfile
    echo 'Content-Transfer-Encoding: binary'                                    >> $tmpfile
    echo ''                                                                     >> $tmpfile
    more $xform_pi                                                              >> $tmpfile
  fi

  if [[ $xform_input != zip:* ]]; then
    echo -e '\r\n--'$boundary                                                   >> $tmpfile
    echo 'Content-ID: <'$xform_input'>'                                         >> $tmpfile
    echo 'Content-Type: application/octet-stream'                               >> $tmpfile
    echo 'Content-Transfer-Encoding: binary'                                    >> $tmpfile
    echo ''                                                                     >> $tmpfile
    more $xform_input                                                           >> $tmpfile
  fi

  for (( i=0 ; i<${#files[@]} ; i++ ))
  do
    echo -e '\r\n--'$boundary                                                   >> $tmpfile
    echo 'Content-ID: <'${files[$i]}'>'                                         >> $tmpfile
    echo 'Content-Type: application/octet-stream'                               >> $tmpfile
    echo 'Content-Transfer-Encoding: binary'                                    >> $tmpfile
    echo ''                                                                     >> $tmpfile
    more ${files[$i]}                                                           >> $tmpfile
  done

  echo -e '\r\n--'$boundary'--'                                                 >> $tmpfile
}

function prepare_xpath_request
{
  echo -e '\r\n\r\n--'$boundary                                                  > $tmpfile
  echo 'Content-Type: text/xml; charset=UTF-8'                                  >> $tmpfile
  echo 'Content-Transfer-Encoding: binary'                                      >> $tmpfile
  echo 'Content-Id: <request>'                                                  >> $tmpfile
  echo ''                                                                       >> $tmpfile
  echo '<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">' >> $tmpfile
  echo '  <soap:Body>'                                                          >> $tmpfile
  echo '    <test:XPathTest xmlns:test="http://www.datapower.com/xslt-test">'   >> $tmpfile
  echo '      <XPathExpr>'$expr'</XPathExpr>'                                   >> $tmpfile
  echo '      <XMLData href="cid:'$xform_input'"/>'                             >> $tmpfile
  echo '    </test:XPathTest>'                                                  >> $tmpfile
  echo '  </soap:Body>'                                                         >> $tmpfile
  echo '</soap:Envelope>'                                                       >> $tmpfile
  echo -e '\r\n--'$boundary                                                     >> $tmpfile
  echo 'Content-ID: <'$xform_input'>'                                           >> $tmpfile
  echo 'Content-Type: text/xml'                                                 >> $tmpfile
  echo 'Content-Transfer-Encoding: binary'                                      >> $tmpfile
  echo ''                                                                       >> $tmpfile
  more $xform_input                                                             >> $tmpfile
  echo -e '\r\n--'$boundary'--'                                                 >> $tmpfile
}

#reading the command line options
while [ "$1" != "" ]; do
  PARAM=$1
  VALUE=$2

  case $PARAM in
    -z ) zip_file=$VALUE
         shift 2;;
    -t ) type=$VALUE
         shift 2;;
    -zx) xform_pi="zip:"$VALUE
         shift 2;;
    -x ) xform_pi=$VALUE
         shift 2;;
    -zi) xform_input="zip:"$VALUE
         shift 2;;
    -i ) xform_input=$VALUE
         shift 2;;
    -a ) xform_mime=$VALUE
         shift 2;;
    -v ) validate_type=$VALUE
         shift 2;;
    -e ) expr=$VALUE
         shift 2;;
    -f ) more_files=$VALUE
         shift 2;;
    -h ) host=$VALUE
         shift 2;;
    -p ) port=$VALUE
         shift 2;;
    -u ) auth=$VALUE
         shift 2;;
    -s ) secured=1
         shift;;
   #-g ) debugged=1
   #     secured=1
   #     shift;;
    -m ) msg_only=1
         shift;;
? | -? ) print_usage
         exit 1 ;;
     * ) echo ">> Unknown parameter '$PARAM'."  >&2
         print_usage
         exit 1 ;;
  esac
done

if [[ $type != 'xslt' ]] && [[ $type != 'ffd' ]] && [[ $type != 'xpath' ]] && [[ $type != 'schema' ]]; then
  echo ">> Invalid request type '$type' is specified."  >&2
  print_usage
  exit 1
fi

if [[ $xform_pi = zip:* ]] || [[ $xform_input = zip:* ]]; then
  if [[ $zip_file = '' ]] || [[ ! -f $zip_file ]]; then
    echo ">> The zip file '$zip_file' cannot be found."  >&2
    print_usage
    exit 1
  fi
fi

if [[ $validate_type != 'schema' ]] && [[ $validate_type != 'wsdl' ]]; then
  echo ">> Invalid type '$validate_type' for schema validation is specified."  >&2
  print_usage
  exit 1
fi

#parsing the required options, and generating the request message
#take care of a xslt request
if [[ $type = 'schema' ]]; then
  if [[ $xform_pi != zip:* ]] && [[ ! -f $xform_pi ]]; then
    echo ">> The schema file '$xform_pi' cannot be found."  >&2
    print_usage
    exit 1
  fi
  if [[ $xform_input != zip:* ]] && [[ ! -f $xform_input ]]; then
    echo ">> The input file '$xform_input' cannot be found."  >&2
    print_usage
    exit 1
  fi

  echo ">> Creating the Schema request..."  >&2
  if [[ $xform_input != '' ]]; then
      prepare_schema_request
  fi
fi

if [[ $type = 'xslt' ]]; then
  if [[ $xform_pi != zip:* ]] && [[ $xform_pi != cid:* ]] && [[ ! -f $xform_pi ]]; then
    echo ">> The stylesheet '$xform_pi' cannot be found."  >&2
    print_usage
    exit 1
  fi
  if [[ $xform_input = '' ]] && [[ ! -f $xform_mime ]]; then
    echo ">> The input file '$xform_input' or the mime file '$xform_mime' cannot be found."  >&2
    print_usage
    exit 1
  fi

  echo ">> Creating the XSLT request..."  >&2
  if [[ $xform_input != '' ]]; then
      prepare_xslt_request
  fi
  if [[ $xform_mime != '' ]]; then
      prepare_xslt_request_swa
  fi
fi

#take care of a ffd request
if [[ $type = 'ffd' ]]; then
  if [[ $xform_pi != zip:* ]] && [[ ! -f $xform_pi ]]; then
    echo ">> The stylesheet '$xform_pi' cannot be found."  >&2
    print_usage
    exit 1
  fi
  if [[ $xform_input != zip:* ]] && [[ ! -f $xform_input ]]; then
    echo ">> The input file '$xform_input' cannot be found."  >&2
    print_usage
    exit 1
  fi

  echo ">> Creating the FFD request..."  >&2
  prepare_ffd_request
fi

#take care of a xpath request
if [[ $type = 'xpath' ]]; then
  if [[ $expr = '' ]] || [[ ! -f $xform_input ]]; then
    echo ">> The xpath is not specified, or the input file '$xform_input' cannot be found."  >&2
    print_usage
    exit 1
  fi

  echo ">> Creating the XPath request..."  >&2
  prepare_xpath_request
fi

#generate message only.
if [[ $msg_only = 1 ]]; then
  echo ">> Message '$tmpfile' is generated. Exit now..."  >&2
  exit 1
fi

#if [[ $debugged = 1 ]] && [[ $auth = '' ]]; then
#  echo ">> Authentication (-u user:passwd) is required when the debug mode is enabled."  >&2
#  print_usage
#  exit 1
#fi

if [[ $host = '' ]]; then
  echo ">> The hostname is not specified. Exit now..."  >&2
  print_usage
  exit 1
fi

#sending the request message over HTTP
if [[ $secured = 0 ]]; then
  #using the default port
  if [[ $port = 0 ]]; then
    port=9990
  fi

  echo ">> Sending the request to http://$host:$port/"  >&2
  if [[ $auth = '' ]]; then
    echo ">> No basic authentication is provided."  >&2
    curl --data-binary @$tmpfile -H 'Content-Type: multipart/related; type="text/xml"; boundary="'$boundary'"' http://$host:$port/
  else
    curl -u "$auth" --data-binary @$tmpfile -H 'Content-Type: multipart/related; type="text/xml"; boundary="'$boundary'"' http://$host:$port/
  fi
fi

#sending the request message over HTTPS
if [[ $secured = 1 ]]; then
  #using the default port
  if [[ $port = 0 ]]; then
    port=9991
  fi

  echo ">> Sending the request to https://$host:$port/"  >&2

# if [[ $debugged = 1 ]]; then
#     curl -k -u "$auth" --data-binary @$tmpfile \
#          -H 'X-DP-INTEROP-TEST-DEBUG: yes' \
#          -H 'Content-Type: multipart/related; type="text/xml"; boundary="'$boundary'"' \
#          https://$host:$port/
# else
#     if [[ $auth = '' ]]; then
#       curl -k --data-binary @$tmpfile -H 'Content-Type: multipart/related; type="text/xml"; boundary="'$boundary'"' https://$host:$port/
#     else
#       curl -k -u "$auth" --data-binary @$tmpfile -H 'Content-Type: multipart/related; type="text/xml"; boundary="'$boundary'"' https://$host:$port/
#     fi
# fi
  if [[ $auth = '' ]]; then
    echo ">> Warning! No basic authentication is provided."  >&2
    curl -k --data-binary @$tmpfile -H 'Content-Type: multipart/related; type="text/xml"; boundary="'$boundary'"' https://$host:$port/
  else
    curl -k -u "$auth" --data-binary @$tmpfile \
         -H 'Content-Type: multipart/related; type="text/xml"; boundary="'$boundary'"' https://$host:$port/
  fi
fi

echo

#cleaning up
rm $tmpfile

